﻿using Proto;
using Proto.Cluster;
using Proto.Cluster.Partition;
using Proto.Cluster.Testing;
using Proto.DependencyInjection;
using Proto.Remote;
using Proto.Remote.GrpcNet;

namespace TestProject;

/// <summary>
/// 액터 시스템 구성
/// </summary>
public static class ActorSystemConfiguration
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Static
    //////////////////////////////////////////////////////////////////////////////// Public

    #region 액터 시스템 추가하기 - AddActorSystem(serviceCollection)

    /// <summary>
    /// 액터 시스템 추가하기
    /// </summary>
    /// <param name="serviceCollection">서비스 컬렉션</param>
    public static void AddActorSystem(this IServiceCollection serviceCollection)
    {
        serviceCollection.AddSingleton
        (
            provider =>
            {
                ActorSystemConfig actorSystemConfig = ActorSystemConfig.Setup();

                GrpcNetRemoteConfig grpcNetRemoteConfig = GrpcNetRemoteConfig
                    .BindToLocalhost()
                    .WithProtoMessages(MessagesReflection.Descriptor);

                ClusterConfig clusterConfig = ClusterConfig.Setup
                (
                    clusterName     : "TestProject",
                    clusterProvider : new TestProvider(new TestProviderOptions(), new InMemAgent()),
                    identityLookup  : new PartitionIdentityLookup()
                )
                .WithClusterKind
                (
                    kind : SmartBulbGrainActor.Kind,
                    prop : Props.FromProducer
                    (
                        () => new SmartBulbGrainActor
                        (
                            (context, clusterIdentity) => new SmartBulbGrain(context, clusterIdentity)
                        )
                    )
                )
                .WithClusterKind
                (
                    kind : SmartHouseGrainActor.Kind,
                    prop : Props.FromProducer
                    (
                        () => new SmartHouseGrainActor
                        (
                            (context, clusterIdentity) => new SmartHouseGrain(context, clusterIdentity)
                        )
                    )
                );

                return new ActorSystem(actorSystemConfig)
                    .WithServiceProvider(provider)
                    .WithRemote(grpcNetRemoteConfig)
                    .WithCluster(clusterConfig);
            }
        );
    }

    #endregion
}