﻿using Proto;
using Proto.Cluster;

namespace TestProject;

/// <summary>
/// 스마트 하우스 그레인
/// </summary>
public class SmartHouseGrain : SmartHouseGrainBase
{
    //////////////////////////////////////////////////////////////////////////////////////////////////// Field
    ////////////////////////////////////////////////////////////////////////////////////////// Private

    #region Field

    /// <summary>
    /// 클러스터 ID
    /// </summary>
    private readonly ClusterIdentity clusterIdentity;

    /// <summary>
    /// 켜진 스마트 전구 정렬 세트
    /// </summary>
    private readonly SortedSet<string> turnedOnSmartBulbSortedSet = new SortedSet<string>();

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 생성자 - SmartHouseGrain(context, clusterIdentity)

    /// <summary>
    /// 생성자
    /// </summary>
    /// <param name="context">컨텍스트</param>
    /// <param name="clusterIdentity">클러스터 ID</param>
    public SmartHouseGrain(IContext context, ClusterIdentity clusterIdentity) : base(context)
    {
        this.clusterIdentity = clusterIdentity;

        Console.WriteLine($"스마트 하우스 그레인 생성 : {this.clusterIdentity.Identity}");
    }

    #endregion

    //////////////////////////////////////////////////////////////////////////////////////////////////// Method
    ////////////////////////////////////////////////////////////////////////////////////////// Public

    #region 스마트 전구 상태 변경하기 - ChangeSmartBulbState(request)

    /// <summary>
    /// 스마트 전구 상태 변경하기
    /// </summary>
    /// <param name="request">스마트 전구 상태 변경 요청</param>
    /// <returns>태스크</returns>
    public override Task ChangeSmartBulbState(ChangeSmartBulbStateRequest request)
    {
        if(request.IsOn)
        {
            this.turnedOnSmartBulbSortedSet.Add(request.SmartBulbIdentity);
        }
        else
        {
            this.turnedOnSmartBulbSortedSet.Remove(request.SmartBulbIdentity);
        }

        Console.WriteLine($"{this.clusterIdentity.Identity} : {this.turnedOnSmartBulbSortedSet.Count}");

        return Task.CompletedTask;
    }

    #endregion
}